/*
 * execute Element: 
 * /triggers
 * /conditions
 * /target
 * /target/singularyZone: '[]'
 * /target/grouped: '[]'
 * /action
 * /action/contentSrc
 * /action/metaData
 * /action/volume
 * /action/loopingTimespan
 * /action/restartOld
 * /action/command
 * /metaData
 * */

Property.setSavedProperty=function (path,value) {
	  Property.setProperty(path, value);
	  Property.setFlag(path, 'ARCHIVE', true);
}

var LOGFILE_NAME = "sonos-controller.log";
var lWarn = {
		logln : function(o) {
		}
	};
	var lInfo = {
		logln : function(o) {
		}
	};


lInfo = new Logger(LOGFILE_NAME);
lWarn = new Logger(LOGFILE_NAME);

function encodeMetaData(metaData) {	
	metaData=metaData.replace(/&/g, '&amp;');
	metaData=metaData.replace(/</g, '&lt;');
	metaData=metaData.replace(/>/g, '&gt;');
	metaData=metaData.replace(/"/g, '&quot;');
	metaData=metaData.replace(/'/g, '&apos;');
	return metaData;
}


function decodeMetaData(metaData) {	
	metaData=metaData.replace( /&amp;/g , '&');
	metaData=metaData.replace(/&lt;/g,'<');
	metaData=metaData.replace(/&gt;/g,'>');
	metaData=metaData.replace(/&quot;/g,'"');
	metaData=metaData.replace(/&apos;/g, "'");
	return metaData;
}

function getPlayerIPForZoneId(zoneID) {
	if (zoneID==null)
		return null;
	var oPlayerArray=Property.getNode('players').getChildren();
	for (var iIndex=0;iIndex<oPlayerArray.length;iIndex++) {
		if (oPlayerArray[iIndex].getChild('zoneID')!=null)
			if (oPlayerArray[iIndex].getChild('zoneID').getValue()==zoneID)
				if (oPlayerArray[iIndex].getChild('disabled').getValue()==false) {
					if (oPlayerArray[iIndex].getChild('coordinator').getValue())
						return oPlayerArray[iIndex].getChild('ip').getValue(); 
					else {
						var groupID=oPlayerArray[iIndex].getChild('group').getValue();						
						for (var jIndex=0;jIndex<oPlayerArray.length;jIndex++) {
							if (oPlayerArray[jIndex].getChild('group').getValue()==groupID)
								if (oPlayerArray[jIndex].getChild('coordinator').getValue())
									return oPlayerArray[jIndex].getChild('ip').getValue();
						}
					}
				}
	}
	return null;
}

function isPlayerForZoneDisabled(zoneID) {
	if (zoneID==null)
		return true;
	var oPlayerArray=Property.getNode('players').getChildren();
	for (var iIndex=0;iIndex<oPlayerArray.length;iIndex++) {
		if (oPlayerArray[iIndex].getChild('zoneID')!=null)
			if (oPlayerArray[iIndex].getChild('zoneID').getValue()==zoneID)
				if (oPlayerArray[iIndex].getChild('disabled').getValue()==false)
					return false;
	}
	return true;
}

function isPlayerForZoneOnline(zoneID) {
	if (zoneID==null)
		return false;
	var oPlayerArray=Property.getNode('players').getChildren();
	for (var iIndex=0;iIndex<oPlayerArray.length;iIndex++) {
		if (oPlayerArray[iIndex].getChild('zoneID')!=null)
			if (oPlayerArray[iIndex].getChild('zoneID').getValue()==zoneID)
				return oPlayerArray[iIndex].getChild('online').getValue();
	}
	return false;
}

function setVolume(path,targetZone) {
	var volume=Property.getProperty(path+"/action/volume");
	
	if (volume!=null) {
		upnpPause(getPlayerIPForZoneId(targetZone));
		upnpSetMute(getPlayerIPForZoneId(targetZone),true);
		upnpSetVolume(getPlayerIPForZoneId(targetZone),volume);
	}
}

function executeCommand(path,targetZone) {
	var actionNode=Property.getNode(path + '/action');
	lInfo.logln('make a ' + actionNode.getChild('command').getValue() + " for " + getPlayerIPForZoneId(targetZone));
	switch (actionNode.getChild('command').getValue()) {
		case 'play':
			upnpPlay(getPlayerIPForZoneId(targetZone));
		break;
		case 'pause':
			upnpPause(getPlayerIPForZoneId(targetZone));
		break;
		case 'mute':
			upnpSetMute(getPlayerIPForZoneId(targetZone),true);
		break;
		case 'unmute':
			upnpSetMute(getPlayerIPForZoneId(targetZone),false);
		break;
		case 'stop':
			upnpStop(getPlayerIPForZoneId(targetZone));
			new Event('sonos.execute.special',{actions:'restoreActualStatusAndGrouping'}).raise();
		break;
		case 'prevTrack':
			var positionInfo=upnpGetCurrentQueue(getPlayerIPForZoneId(targetZone));
			var iIndex=positionInfo.trackPos;
			if (iIndex>0)
				iIndex--;
			upnpSeek(getPlayerIPForZoneId(targetZone),true,iIndex);
		break;
		case 'nextTrack':
			var positionInfo=upnpGetCurrentQueue(getPlayerIPForZoneId(targetZone));
			upnpSeek(getPlayerIPForZoneId(targetZone),true,positionInfo.trackPos+1);
		break;
	}
}

function executePlaylist(path,targetZone) {
	var actionNode=Property.getNode(path + '/action');
	var contentSrc=actionNode.getChild('contentSrc').getValue();
	lInfo.logln('play a playlist ' + contentSrc + " on "+ getPlayerIPForZoneId(targetZone));
	//upnpLoadPlaylist(getPlayerIPForZoneId(targetZone),contentSrc,"");

	var metaData="";
	if (actionNode.getChild('metaData')!=null)
		metaData=actionNode.getChild('metaData').getValue();

	var queueID=upnpCreateQueue(getPlayerIPForZoneId(targetZone));
	var queueURI=upnpGetQueueURI(getPlayerIPForZoneId(targetZone),queueID);
	upnpSetContentSrc(getPlayerIPForZoneId(targetZone),queueURI,"");
	upnpAddUriToQueue(getPlayerIPForZoneId(targetZone),queueID,contentSrc,metaData);
	
	upnpPlay(getPlayerIPForZoneId(targetZone));
	upnpSetMute(getPlayerIPForZoneId(targetZone),false);
	upnpSetPlayMode(getPlayerIPForZoneId(targetZone), "NORMAL");
}

function executeFavorite(path,targetZone) {
	var actionNode=Property.getNode(path + '/action');
	var contentSrc=actionNode.getChild('contentSrc').getValue();
	var metaData="";
	if (actionNode.getChild('metaData')!=null)
		metaData=actionNode.getChild('metaData').getValue();
	if (contentSrc.indexOf('x-sonosapi-stream')<0) {
		/*if (actionNode.getChild('singleTrack').getValue()==true) {			
			metaData=encodeMetaData(unescape(metaData));
			upnpSetContentSrc(getPlayerIPForZoneId(targetZone),contentSrc,metaData);
		} else {*/
		lInfo.logln('new Metadata ' + metaData);
			var queueID=upnpCreateQueue(getPlayerIPForZoneId(targetZone));
			upnpAddUriToQueue(getPlayerIPForZoneId(targetZone),queueID,contentSrc,metaData);
			var queueURI=upnpGetQueueURI(getPlayerIPForZoneId(targetZone),queueID);
			upnpSetContentSrc(getPlayerIPForZoneId(targetZone),queueURI,"");			
		//}
	} else {
		upnpSetRadio(getPlayerIPForZoneId(targetZone),encodeMetaData(contentSrc),getXMLItem(decodeMetaData(unescape(metaData)),'dc:title'));
	}
	upnpPlay(getPlayerIPForZoneId(targetZone));
	upnpSetMute(getPlayerIPForZoneId(targetZone),false);
	upnpSetPlayMode(getPlayerIPForZoneId(targetZone), "NORMAL");
}

function executePlainRadio(path,targetZone) {
	var actionNode=Property.getNode(path + '/action');
	var contentSrc=actionNode.getChild('contentSrc').getValue();
	var title=actionNode.getChild('info/title').getValue();
	upnpSetRadio(getPlayerIPForZoneId(targetZone),encodeMetaData(contentSrc),title);
	upnpPlay(getPlayerIPForZoneId(targetZone));
	upnpSetMute(getPlayerIPForZoneId(targetZone),false);
	upnpSetPlayMode(getPlayerIPForZoneId(targetZone), "NORMAL");
}

function executeQueue(path,targetZone) {
	var actionNode=Property.getNode(path + '/action');
	var metaData="";
	var oQueueArray=actionNode.getChild('queue').getChildren();
	var queueID=upnpCreateQueue(getPlayerIPForZoneId(targetZone));
	for (var iIndex=0;iIndex<oQueueArray.length;iIndex++) {
		var contentSrc=oQueueArray[iIndex].getChild('contentSrc').getValue();
		upnpAddUriToQueue(getPlayerIPForZoneId(targetZone),queueID,contentSrc,'');
	}
	var queueURI=upnpGetQueueURI(getPlayerIPForZoneId(targetZone),queueID);
	upnpSetContentSrc(getPlayerIPForZoneId(targetZone),queueURI,"");
	upnpPlay(getPlayerIPForZoneId(targetZone));
	upnpSetMute(getPlayerIPForZoneId(targetZone),false);
	upnpSetPlayMode(getPlayerIPForZoneId(targetZone), "NORMAL");
}

function executePlainSong(path,targetZone) {
	var actionNode=Property.getNode(path + '/action');
	var contentSrc=actionNode.getChild('contentSrc').getValue();
	var metaData="";
	upnpSetContentSrc(getPlayerIPForZoneId(targetZone),contentSrc.replace(/&/g, '&amp;'),metaData);
	upnpPlay(getPlayerIPForZoneId(targetZone));
	upnpSetMute(getPlayerIPForZoneId(targetZone),false);
	upnpSetPlayMode(getPlayerIPForZoneId(targetZone), "NORMAL");
}

function executeLibrary(path,targetZone) {
	var actionNode=Property.getNode(path + '/action');
	var contentSrc=actionNode.getChild('contentSrc').getValue();
	var metaData="";
	upnpSetContentSrc(getPlayerIPForZoneId(targetZone),contentSrc,metaData);
	upnpPlay(getPlayerIPForZoneId(targetZone));
	upnpSetMute(getPlayerIPForZoneId(targetZone),false);
	upnpSetPlayMode(getPlayerIPForZoneId(targetZone), "NORMAL");	
}


function fetchStructure() {
	if (Property.getNode('players')!=null) {
		var playersArray=Property.getNode('players').getChildren();
		for (var iIndex=0;iIndex<playersArray.length;iIndex++) {
			var sPlayerIP=playersArray[iIndex].getChild('ip').getValue();
			var oActualStructure=upnpGetSonosGroups(sPlayerIP);
			if (oActualStructure!=null)
				return oActualStructure;
		}
	}
	return null;
}

function groupPlayers(path) {
	

	var oActualStructure=fetchStructure();
	var activeCoordinator=null;
	var activeSubstructure=[];
	if (Property.getNode('groupInfo/dsGroup')==null) {
		lInfo.logln('no grouping from dS active');
		Property.setSavedProperty('groupInfo/customerStructure',JSON.stringify(oActualStructure));
	} else {
		var sGroupID=Property.getNode('groupInfo/dsGroup').getValue();
		lInfo.logln('a dS group is active');
		for (var iIndex=0;iIndex<oActualStructure.length;iIndex++) {
			if (oActualStructure[iIndex].id==sGroupID) {
				lInfo.logln('found a dS group');
				activeCoordinator=oActualStructure[iIndex].coordinator;
				activeSubstructure=oActualStructure[iIndex].member;
			}
		}
	}
	var targetDevices=[];
	var oGroupArray=Property.getNode(path+'/target/grouped').getChildren();
	var oPlayerArray=Property.getNode('players').getChildren();
	for (var iIndex=0;iIndex<oGroupArray.length;iIndex++) {
		for (var jIndex=0;jIndex<oPlayerArray.length;jIndex++) {
			if (oPlayerArray[jIndex].getChild('zoneID')!=null) 
				if (oPlayerArray[jIndex].getChild('disabled')!=null) 
					if (oPlayerArray[jIndex].getChild('online')!=null) 
						if (oPlayerArray[jIndex].getChild('online').getValue()==true)
							if (oPlayerArray[jIndex].getChild('disabled').getValue()==false) {
								if (oGroupArray[iIndex].getValue()==0) {
									targetDevices.push(oPlayerArray[jIndex].getChild('id').getValue());
								} else
									if (oPlayerArray[jIndex].getChild('zoneID').getValue()==oGroupArray[iIndex].getValue())
										targetDevices.push(oPlayerArray[jIndex].getChild('id').getValue());
							}
		}
	}
	
	lInfo.logln('should be executed on devices ' + JSON.stringify(targetDevices))
	if ((activeCoordinator==null) || (targetDevices.indexOf(activeCoordinator)<0)) {
		lInfo.logln('should buildup a new group because no ds group is present or active dS-group coordinator is not a target for message');
		activeCoordinator=targetDevices[0];		
		lInfo.logln('set as coordinator ' + activeCoordinator);
		upnpSetPlayerAsStandAloneGroup(Property.getProperty('players/'+activeCoordinator+'/ip'));							
		oActualStructure=fetchStructure();
		for (var iIndex=0;iIndex<oActualStructure.length;iIndex++) {
			if (oActualStructure[iIndex].coordinator==activeCoordinator) {
				Property.setSavedProperty('groupInfo/dsGroup',oActualStructure[iIndex].id);
			}
		}
	} else {
		lInfo.logln('on existing dSGroup there is a suitable coordinator, so maybe only removing or adding devices is needed');
		
	}
	
	
	for ( var jIndex = 0; jIndex < targetDevices.length; jIndex++) {
		if (targetDevices[jIndex]!=activeCoordinator) {
			var fShouldAdd=true;
			for (var iIndex=0;iIndex<activeSubstructure.length;iIndex++) {
				if (targetDevices[jIndex]==activeSubstructure[iIndex].id)
					fShouldAdd=false;
			}
			if (fShouldAdd)
				addPlayerToGroup(Property.getProperty('players/'+targetDevices[jIndex]+'/ip'),activeCoordinator);	
		}
			
			
	}
	return Property.getProperty('players/'+activeCoordinator+'/zoneID');
}

function addPlayerToGroup(memberIP,coordinatorID) {
	lInfo.logln('adding player '  +memberIP +' to ' + coordinatorID)
	upnpSetContentSrc(memberIP, "x-rincon:" + coordinatorID);
}

function groupMembers(targetStructure,referenceStructure) {
	if (referenceStructure == null) {
		lInfo.logln('group is not existing before, so just skipping a guessing');
		for (var iIndex=0;iIndex<targetStructure.member.length;iIndex++) {
			if (targetStructure.coordinator!=targetStructure.member[iIndex].id)
				addPlayerToGroup(targetStructure.member[iIndex].ip,targetStructure.coordinator);
		}
	} else {
		lInfo.logln('group exist ... somewhat... at the moment the coordinator is the same');
		for (var iIndex=0;iIndex<targetStructure.member.length;iIndex++) {
			var iMemberID=targetStructure.member[iIndex].id;
			var fReferenceIsInTarget=false;
			for (var jIndex=0;jIndex<referenceStructure.member.length;jIndex++) {
				if (referenceStructure.member[jIndex].id==iMemberID)
					ReferenceIsInTarget=true;
			}
			if (!fReferenceIsInTarget)
				if (targetStructure.coordinator!=targetStructure.member[iIndex].id)
					addPlayerToGroup(targetStructure.member[iIndex].ip,targetStructure.coordinator);
		}
		
	}
}

function ungroup() {
	if (Property.getProperty('groupInfo/customerStructure')==null) {
		Property.getNode('/scripts/sonos-controller').removeChild('groupInfo');
		return;
	}
		
	lInfo.logln('restore customer grouping ' + Property.getProperty('groupInfo/customerStructure'));
	var formerGrouping = JSON.parse(Property.getProperty('groupInfo/customerStructure'));
	var actualGrouping = fetchStructure();
	for (var iIndex=0;iIndex<formerGrouping.length;iIndex++) {
		var oGroupToRestore=formerGrouping[iIndex];
		var fGroupExisting=false;
		for (var jIndex=0;jIndex<actualGrouping.length;jIndex++) {
			if (actualGrouping[jIndex].coordinator==oGroupToRestore.coordinator) {
				lInfo.logln('the coordinator is coordinator of a group, so we reuse this group');
				fGroupExisting=true;
				groupMembers(oGroupToRestore,actualGrouping[jIndex]);
			}
		}
		if (!fGroupExisting) {
			lInfo.logln('grouping is not existent, just buildup it freshly');
			upnpSetPlayerAsStandAloneGroup(Property.getProperty('players/'+oGroupToRestore.coordinator+'/ip'));
			upnpPause(Property.getProperty('players/'+oGroupToRestore.coordinator+'/ip'));
			groupMembers(oGroupToRestore);
		}
	}
	if (Property.getNode('groupInfo/groupScheduleID')!=null) {
		lInfo.logln('!! removing a scheduled ungrouping event !!');
	    var propNode = Property.getNode('/system/EventInterpreter/ScheduledEvents/' + Property.getProperty('groupInfo/groupScheduleID'));
	    if (propNode) {
	        propNode.getParent().removeChild(propNode);
	    }
	}
	Property.getNode('/scripts/sonos-controller').removeChild('groupInfo');
	
}

function executePath(path) {	
	try {
		lInfo.logln('starting execute ' + path);
		var actionNode=Property.getNode(path + '/action');
		var targetZone=[];
		var fGrouped=false;
		var resetAfterPlay=false;
		var aInactives=[];
		if (Property.getNode(path+"/target/singularyZone")!=null) {
			var oNodeArray=Property.getNode(path+"/target/singularyZone").getChildren();
			for (var iIndex=0;iIndex<oNodeArray.length;iIndex++) {
				if (oNodeArray[iIndex].getValue()==0) {
					// all devices!
					var oPlayerArray=Property.getNode('players').getChildren();
					for (var jIndex=0;jIndex<oPlayerArray.length;jIndex++) {
						if (oPlayerArray[jIndex].getChild('zoneID')!=null)
							if (oPlayerArray[jIndex].getChild('zoneID').getValue()!=0)
								if (oPlayerArray[jIndex].getChild('disabled')!=null)
									if (oPlayerArray[jIndex].getChild('disabled').getValue()==false)
										if (oPlayerArray[jIndex].getChild('online')!=null) 
											if (oPlayerArray[jIndex].getChild('online').getValue()==true) {
												if (oPlayerArray[jIndex].getChild('coordinator')!=null) 
													if (oPlayerArray[jIndex].getChild('coordinator').getValue()==true) 
														targetZone.push(oPlayerArray[jIndex].getChild('zoneID').getValue()); 
											} else {
												aInactives.push(oPlayerArray[jIndex].getChild('zoneID').getValue());
											}
											
					}
				} else {
					if (isPlayerForZoneOnline(oNodeArray[iIndex].getValue()))
						targetZone.push(oNodeArray[iIndex].getValue());
					else
						aInactives.push(oNodeArray[iIndex].getValue());
				}
					
			}
		} else {
			var oNodeArray=Property.getNode(path+"/target/grouped").getChildren();
			for (var iIndex=0;iIndex<oNodeArray.length;iIndex++) {
				if (oNodeArray[iIndex].getValue()==0) {
					var oPlayerArray=Property.getNode('players').getChildren();
					for (var jIndex=0;jIndex<oPlayerArray.length;jIndex++) {
						if (oPlayerArray[jIndex].getChild('zoneID')!=null)
							if (oPlayerArray[jIndex].getChild('zoneID').getValue()!=0)
								if (oPlayerArray[jIndex].getChild('disabled')!=null)
									if (oPlayerArray[jIndex].getChild('disabled').getValue()==false)
										if (oPlayerArray[jIndex].getChild('online')!=null) 
											if (oPlayerArray[jIndex].getChild('online').getValue()==true) {
												/*if (oPlayerArray[jIndex].getChild('coordinator')!=null) grouping will make a new coordinator
													if (oPlayerArray[jIndex].getChild('coordinator').getValue()==true)*/
														targetZone.push(oPlayerArray[jIndex].getChild('zoneID').getValue())
											} else {
												aInactives.push(oPlayerArray[jIndex].getChild('zoneID').getValue());
											}
					}
				} else {
					if (isPlayerForZoneOnline(oNodeArray[iIndex].getValue()))
						targetZone.push(oNodeArray[iIndex].getValue());
					else
						aInactives.push(oNodeArray[iIndex].getValue());
				}
			}
			fGrouped=true;
			resetAfterPlay=true;
		}
	
		if (targetZone.length==0) {
			
			if (aInactives.length>0)
				if (raisedEvent.parameter.retry==null) {
					lInfo.logln('nothing can be done but there were inactive players ... so just fetch the structure and retry it');
					new Event('sonos.player',{actions:'updatePlayer'}).raise();
					raisedEvent.parameter.retry=true;
					raisedEvent.parameter.time="+1";
					new Event(raisedEvent.name,raisedEvent.parameter).raise();
				}
			lInfo.logln('no (active) player found, so just breaking');	
			return;
		}
		var fRestore=false;
		var fRestart=false;
		var fTimespan=false;
		if (Property.getProperty(path+"/action/restoreAfter")!=null)
			fRestore=Property.getProperty(path+"/action/restoreAfter");
		if (Property.getProperty(path+"/action/restartAfter")!=null)
			fRestart=Property.getProperty(path+"/action/restartAfter");

		if (actionNode.getChild('loopingTimespan')!=null) {
			fRestore=true;
			fTimespan=true;
		}
		if (actionNode.getChild('maxDuration')!=null) {
			fRestore=true;
			fTimespan=true;
		}
		var aZonesWhichArePreserved=[];
		if (fRestore) {
			for (var iIndex=0;iIndex<targetZone.length;iIndex++) {
				if (preserveStatus(targetZone[iIndex]))
					aZonesWhichArePreserved.push(targetZone[iIndex]);
			}
		}
		
		
		// Step 1: stop and set volume if needed for each target
		if (Property.getProperty(path+"/action/volume")!=null)		
			for (var iIndex=0;iIndex<targetZone.length;iIndex++)
				if (!isPlayerForZoneDisabled(targetZone[iIndex]))
					setVolume(path,targetZone[iIndex]);
		
		// Step 2: execute the action for each target
		if (fGrouped) 
			targetZone=[groupPlayers(path)];
			
		for (var iIndex=0;iIndex<targetZone.length;iIndex++) {
			lInfo.logln('command is ' + actionNode.getChild('type').getValue() + ' for target ' + targetZone[iIndex]);
			if (!isPlayerForZoneDisabled(targetZone[iIndex]))
				switch (actionNode.getChild('type').getValue()) {
					case 'command':
						executeCommand(path,targetZone[iIndex]);
						break;
					case 'playlist':		
						executePlaylist(path,targetZone[iIndex]);
						break;
					case 'favorite':
						executeFavorite(path,targetZone[iIndex]);
						break;
					case 'plainSong':
						executePlainSong(path,targetZone[iIndex]);
						break;
					case 'plainRadio':
						executePlainRadio(path,targetZone[iIndex]);
						break;
					case 'plainQueue':
						executeQueue(path,targetZone[iIndex]);
						break;
					case 'library':
						executeLibrary(path,targetZone[iIndex]);
						break;
					case 'text2speech':
						executeLibrary(path,targetZone[iIndex]);
						break;
				}
		}
	
		
		var deltaTime=null;
		if (actionNode.getChild('loopingTimespan')!=null) {
			deltaTime=parseInt(''+actionNode.getChild('loopingTimespan').getValue(),10);
		} 
		
		if (resetAfterPlay) {
			if ((actionNode.getChild('loopingTimespan')!=null) && (actionNode.getChild('maxDuration')==null)) {
				for (var iIndex=0;iIndex<targetZone.length;iIndex++) {
					upnpSetPlayMode(getPlayerIPForZoneId(targetZone[iIndex]), 'REPEAT_ALL');
				}
			} else {			
				if (actionNode.getChild('info/length')!=null)
					deltaTime=parseInt('' + actionNode.getChild('info/length').getValue(),10)+1;
				else {
					var response=upnpGetPositionInfo(getPlayerIPForZoneId(targetZone[0]));				
					deltaTime=parseInt('' + timeFromDuration(getXMLItem(response, 'TrackDuration')),10) + 1; //+1 tribute on dss11e
				}
									
			}
			if (actionNode.getChild('maxDuration')!=null)
				if (actionNode.getChild('maxDuration').getValue()<(deltaTime-1))
					deltaTime=parseInt('' + actionNode.getChild('maxDuration').getValue(),10)+1;

			if (deltaTime>1) {
				lInfo.logln('ungroup after ' + deltaTime + ' sek');
				if (fGrouped) {
					var subscriptionID=""+Math.random();			
					Property.setProperty('groupInfo/subscriptionID',""+subscriptionID);
					var scheduleID=new TimedEvent ('sonos.execute.special','+' + deltaTime,{actions:'ungroup',subscriptionID:subscriptionID}).raise();
					Property.setProperty('groupInfo/groupScheduleID',scheduleID);
				}
			} else {
				lInfo.logln('maybe we want to ungroup, but we have no real deltatime so no ungroup will be done and preserved status is deleted');
				for (var iIndex=0;iIndex<aZonesWhichArePreserved.length;iIndex++) {

					var oPlayerArray=Property.getNode('players').getChildren();
					for (var jIndex=0;jIndex<oPlayerArray.length;jIndex++) {
						if (oPlayerArray[jIndex].getChild('zoneID')!=null)
							if (oPlayerArray[jIndex].getChild('zoneID').getValue()==aZonesWhichArePreserved[iIndex])
								oPlayerArray[jIndex].removeChild('restore');
					}

				}
			}
				
		}
		if (fRestore) {
			if (deltaTime==null) {
				if (actionNode.getChild('info/length')!=null)
					deltaTime=parseInt('' + actionNode.getChild('info/length').getValue(),10)+1;
				else {
					var response=upnpGetPositionInfo(getPlayerIPForZoneId(targetZone[0]));
					deltaTime=parseInt('' + timeFromDuration(getXMLItem(response, 'TrackDuration'))) + 1; //+1 tribute on dss11e
				}
					
				if (actionNode.getChild('maxDuration')!=null)
					if (actionNode.getChild('maxDuration').getValue()<(deltaTime-1))
						deltaTime=parseInt('' + actionNode.getChild('maxDuration').getValue(),10)+1;
							
			}
			if (deltaTime>1) {
				lInfo.logln('reset after ' + deltaTime + ' sek');	
				var subscriptionID=""+Math.random();			
				Property.setProperty('resettingSubscriptionID',""+subscriptionID);
				var scheduleID=new TimedEvent ('sonos.execute.special','+' + deltaTime,{actions:'restoreActualStatus',subscriptionID:subscriptionID,restartAfter:fRestart}).raise();	
				Property.setProperty('resetScheduleID',scheduleID);
			} else {
				lInfo.logln('maybe we want to reset, but we have no real deltatime so no ungroup will be done and preserved status is deleted');
				for (var iIndex=0;iIndex<aZonesWhichArePreserved.length;iIndex++) {

					var oPlayerArray=Property.getNode('players').getChildren();
					for (var jIndex=0;jIndex<oPlayerArray.length;jIndex++) {
						if (oPlayerArray[jIndex].getChild('zoneID')!=null)
							if (oPlayerArray[jIndex].getChild('zoneID').getValue()==aZonesWhichArePreserved[iIndex])
								oPlayerArray[jIndex].removeChild('restore');
					}

				}
			}
		}
		
	} catch (e) {
		lWarn.logln('error ' + e);
	}
}

function preserveStatus(zoneID) {
	zoneID=parseInt('' + zoneID,10);
	lInfo.logln('preserve Status for zone ' + zoneID);
	var path="";
	var oPlayerArray=Property.getNode('players').getChildren();
	for (var jIndex=0;jIndex<oPlayerArray.length;jIndex++) {
		if (oPlayerArray[jIndex].getChild('zoneID')!=null)
			if (oPlayerArray[jIndex].getChild('zoneID').getValue()==zoneID)
				path=oPlayerArray[jIndex].getPath();
	}
	if (Property.getNode(path + '/restore')==null) { 
	
		// get Playmode, Volume, content
		var oldVolumne=upnpGetVolume(getPlayerIPForZoneId(zoneID));
		var fOldMute=upnpGetMute(getPlayerIPForZoneId(zoneID));
		var sStringMediaInfo=upnpGetMediaInfo(getPlayerIPForZoneId(zoneID));
		var currentTransportMode=upnpGetTransportInfo(getPlayerIPForZoneId(zoneID));
		var sStringPositionInfo=upnpGetPositionInfo(getPlayerIPForZoneId(zoneID));
		var currentPlayMode = upnpGetPlayMode(getPlayerIPForZoneId(zoneID));
		var currentUri=getXMLItem(sStringMediaInfo,'CurrentURI');
		var currentMeta=getXMLItem(sStringMediaInfo,'CurrentURIMetaData');
		
		var sTrackNumber=getXMLItem(sStringPositionInfo,'Track');
		var sDuration=getXMLItem(sStringPositionInfo,'RelTime');
		
		
		if (currentUri.indexOf('sonosapi-stream')!=-1) {
			currentMeta=unescapeXML(getXMLItem(unescapeXML(currentMeta), 'dc:title'));
		}
		
		lInfo.logln('oldVolumne ' + oldVolumne);
		lInfo.logln('fOldMute ' + fOldMute);
		lInfo.logln('currentUri ' + currentUri);
		lInfo.logln('currentMeta ' + currentMeta);	
		lInfo.logln('currentTrack ' + sTrackNumber);
		lInfo.logln('currentDuration ' + sDuration);
		lInfo.logln('currentTransportMode ' + currentTransportMode);
		lInfo.logln('currentPlayMode ' + currentPlayMode);
		Property.setProperty(path + '/restore/volume',oldVolumne);
		Property.setProperty(path + '/restore/mute',fOldMute);
		Property.setProperty(path + '/restore/uri',currentUri);
		Property.setProperty(path + '/restore/meta',currentMeta);
		Property.setProperty(path + '/restore/track',sTrackNumber);
		Property.setProperty(path + '/restore/duration',sDuration);
		Property.setProperty(path + '/restore/playMode',currentPlayMode);
		Property.setProperty(path + '/restore/currentTransportMode',currentTransportMode);
		return true;
	} else
		return false;
}

function restoreStatus(zoneID) {
	
	var path="";
	var oPlayerArray=Property.getNode('players').getChildren();
	for (var jIndex=0;jIndex<oPlayerArray.length;jIndex++) {
		if (oPlayerArray[jIndex].getChild('zoneID')!=null)
			if (oPlayerArray[jIndex].getChild('zoneID').getValue()==zoneID)
				path=oPlayerArray[jIndex].getPath();
	}
	if (Property.getNode(path + '/restore')!=null) {
		if (Property.getProperty(path+'/restore/mute')) {
			lInfo.logln('mute is active, so make this first');
			upnpSetMute(getPlayerIPForZoneId(zoneID),Property.getProperty(path+'/restore/mute'));
		} 
		upnpPause(getPlayerIPForZoneId(zoneID));
		upnpSetVolume(getPlayerIPForZoneId(zoneID),Property.getProperty(path+'/restore/volume'));
		if (Property.getProperty(path+'/restore/uri').indexOf('sonosapi-stream')!=-1)
			upnpSetRadio(getPlayerIPForZoneId(zoneID),Property.getProperty(path+'/restore/uri'),Property.getProperty(path+'/restore/meta'));
		else
			upnpSetContentSrc(getPlayerIPForZoneId(zoneID),Property.getProperty(path+'/restore/uri'),Property.getProperty(path+'/restore/meta'));

		upnpSeek(getPlayerIPForZoneId(zoneID), true, Property.getProperty(path+'/restore/track'))
		upnpSeek(getPlayerIPForZoneId(zoneID), false, Property.getProperty(path+'/restore/duration'))
		
		if (!Property.getProperty(path+'/restore/mute')) {
			lInfo.logln('mute is not active, so make this first');
			upnpSetMute(getPlayerIPForZoneId(zoneID),Property.getProperty(path+'/restore/mute'));
		}
		upnpSetPlayMode(getPlayerIPForZoneId(zoneID),Property.getProperty(path+'/restore/playMode'));
		
		if (raisedEvent.parameter.restartAfter!=null)
			if (raisedEvent.parameter.restartAfter)
				if (Property.getNode(path+'/restore/currentTransportMode')!=null){
					if (Property.getProperty(path+'/restore/currentTransportMode')=='PLAYING') {
						upnpPlay(getPlayerIPForZoneId(zoneID));
					} else
					if (Property.getProperty(path+'/restore/currentTransportMode')=='STOPPED') {
						upnpStop(getPlayerIPForZoneId(zoneID));
					} else
						upnpPause(getPlayerIPForZoneId(zoneID));
				}
		Property.getNode(path).removeChild('restore');	

	}
	
}

lInfo.logln("-----------------------------");
lInfo.logln(JSON.stringify(raisedEvent));

if (raisedEvent.name == 'sonos.execute') {
	executePath(unescape(raisedEvent.parameter.path));
	new Event("sonos.saved", {actions:'execute'}).raise();;	
}
if (raisedEvent.name == 'sonos.execute.special') {
	if (raisedEvent.parameter.actions=='ungroup') {
		if (raisedEvent.parameter.subscriptionID==null)
			ungroup();
		else {
			if(Property.getNode('groupInfo/subscriptionID')!=null)
				if (Property.getProperty('groupInfo/subscriptionID')==raisedEvent.parameter.subscriptionID)
					ungroup();
		}
	}
	if (raisedEvent.parameter.actions=='preserveActualStatus') {
		if (raisedEvent.parameter.zoneID!=null) {
			lInfo.logln('preserve Status from zone ' + raisedEvent.parameter.zoneID);
			preserveStatus(raisedEvent.parameter.zoneID);
		} else {
			if(Property.getNode('resettingSubscriptionID')!=null)
				if (Property.getProperty('resettingSubscriptionID')==raisedEvent.parameter.subscriptionID) {
					var oPlayerArray=Property.getNode('players').getChildren();
					for (var jIndex=0;jIndex<oPlayerArray.length;jIndex++) {
						if (oPlayerArray[jIndex].getChild('zoneID')!=null)
							if (oPlayerArray[jIndex].getChild('restore')!=null) {
								preserveStatus(oPlayerArray[jIndex].getChild('zoneID').getValue());
							} 
					}
					
				}
		} 
	}
	if (raisedEvent.parameter.actions=='restoreActualStatus') {
		if (raisedEvent.parameter.zoneID!=null) {
			lInfo.logln('restore Status from zone ' + raisedEvent.parameter.zoneID);
			restoreStatus(raisedEvent.parameter.zoneID);
		} else {
			if(Property.getNode('resettingSubscriptionID')!=null)
				if (Property.getProperty('resettingSubscriptionID')==raisedEvent.parameter.subscriptionID) {
					var oPlayerArray=Property.getNode('players').getChildren();
					for (var jIndex=0;jIndex<oPlayerArray.length;jIndex++) {
						if (oPlayerArray[jIndex].getChild('zoneID')!=null)
							if (oPlayerArray[jIndex].getChild('restore')!=null) {
								restoreStatus(oPlayerArray[jIndex].getChild('zoneID').getValue());
							} 
					}
					if (Property.getNode('resetScheduleID')!=null) {
						lInfo.logln('!! removing a scheduled reseting event !!');
					    var propNode = Property.getNode('/system/EventInterpreter/ScheduledEvents/' + Property.getProperty('resetScheduleID'));
					    if (propNode) {
					        propNode.getParent().removeChild(propNode);
					    }
					}
				}
		}
	}
	if (raisedEvent.parameter.actions=='restoreActualStatusAndGrouping') {
		if (Property.getNode('players')!=null) {
			ungroup();
			var oPlayerArray=Property.getNode('players').getChildren();
			for (var jIndex=0;jIndex<oPlayerArray.length;jIndex++) {
				if (oPlayerArray[jIndex].getChild('zoneID')!=null)
					if (oPlayerArray[jIndex].getChild('restore')!=null) {
						restoreStatus(oPlayerArray[jIndex].getChild('zoneID').getValue());
					} 
			}
			if (Property.getNode('resetScheduleID')!=null) {
				lInfo.logln('!! removing a scheduled reseting event !!');
			    var propNode = Property.getNode('/system/EventInterpreter/ScheduledEvents/' + Property.getProperty('resetScheduleID'));
			    if (propNode) {
			        propNode.getParent().removeChild(propNode);
			    }
			}
		}
	}
}
