/**
 * @file main.js
 * 
 * @brief This file contains implementation of User interface for the Sonos controller app.
 * 
 * @author: mobile2DS
 * 
 * @date: 31.1.2014
 */

var iSubscript;
var fSearchInProgress=false;
function startListener() {

	iSubscript = Math.floor(Math.random() * 100000000000);
	dss.ajaxSyncRequest('/json/event/subscribe', {
	    subscriptionID : iSubscript,
	    name : 'sonos.saved'
	});
	refreshListener();
}

var fChangedCallback=[];

function refreshListener() {
	dss.ajaxAssyncRequestWithoutAnswer('/json/event/get', {
	    subscriptionID : iSubscript,
	    timeout : 5 * 1000
	}, function(answer) {	
        try {
            var result = Ext.JSON.decode(answer);
    		if (result.ok==false) {
    			startListener();
				return;
            }
    		
    		if (result.result != null) {
    			for (var iIndex=0;iIndex<result.result.events.length;iIndex++) {
    				var oItem=result.result.events[iIndex].properties;
					for (var jIndex=0;jIndex<fChangedCallback.length;jIndex++)
						fChangedCallback[jIndex](oItem);
    			}			
    		}	
		} catch (e) {}
		refreshListener();
	});
}
var presetStore;
var activityStore;
var playerStore;

var fLoading = false;
var maxTextLength = 300;
var mediaLibrary = null;

var mediaServiceIsConnected = false;

function getFileObjectFromMedialib(guid, parent) {
	if (parent == null)
		parent = mediaLibrary;
	for (var iIndex = 0; iIndex < parent.files.length; iIndex++) {
		if (parent.files[iIndex].id == guid)
			return parent.files[iIndex];
	}
	for (var iIndex = 0; iIndex < parent.child.length; iIndex++) {
		var oReturn = getFileObjectFromMedialib(guid, parent.child[iIndex]);
		if (oReturn != null)
			return oReturn;
	}
	return null;
}

Ext.apply(Ext.data.SortTypes, {
    asSonosAction: function(oValue) {
    	var baseIndex=0;
        switch(oValue.type) {
			case 'plainSong':
	        	baseIndex=1;
			break;
			case 'plainRadio':
	        	baseIndex=2;
			break;
			case 'plainQueue':
	        	baseIndex=3;
			break;
			case 'favorite':
	        	baseIndex=4;
			break;
			case 'playlist':		
		    	baseIndex=5;
			break;
			case 'library':
	        	baseIndex=6;
			break;
			case 'text2speech':
	        	baseIndex=7;
			break;
	        case 'command':
	        	baseIndex=8;
			break;
        }      
        
        return baseIndex*10000;
    }
});


dss.getPropertyInteger = function(sProp) {
	var sURL = '/json/property/getInteger';
	var sPfad = escape(sProp);
	var sReturn = dss.ajaxSyncRequest(sURL, {
		'path' : sPfad
	});
	var obj = eval('(' + sReturn + ')');
	if (obj.ok)
		return obj.result.value;
	else
		return null;
};

Ext.define('DSS.addon.SonosController', {
	extend : 'DSS.addon.Framework',
	fExpertMode : false,
	config : {
		appName : 'Sonos Controller',
		appId : 'sonos-controller-app',
		appVersion : '0.8.2',
		appIcon : 'default_icon.png'
	},

	getHelp : function() {
		return Ext.create('Ext.Component', {
			whiteSpace : 'normal',
			padding : '0 20 0 0', // top right bottom left
			loader : {
				url : 'locale/' + dss.staticDataModel.activeLanguage + '/help.html',
				autoLoad : true,
				disableCaching : false
			}
		});
	},

	discover : null,
	discoveryDialog : null,
	sNetworkIP : '', // dSS network IP
	bIPFound : false,
	playersGrid : null,
	activitiesGrid : null,
	oPresets : null,
	oMainActivities : null,
	oPlayers : null,
	mainAddonPanel : null,
	presetsGrid : null,
	arrayMediaLibrary : null,
	arraySpeechSettings : null,
	lastConfigurationUpdate : 0,
	filterZone : [],
	loadSpeechData : function() {
		maxTextLength = dss.getPropertyInteger('/scripts/sonos-controller/media-service/speech/maxTextLength');
		if (Ext.data.StoreManager.lookup('txt2speech_langs') == null) {
			Ext.create('Ext.data.Store', {
				storeId : 'txt2speech_langs',
				fields : [ 'langCode', 'langName' ],
				proxy : {
					type : 'memory',
					reader : {
						type : 'json',
						root : 'items'
					}
				}
			});
		} else
			Ext.data.StoreManager.lookup('txt2speech_langs').removeAll();

		if (Ext.data.StoreManager.lookup('txt2speech_voices') == null) {
			Ext.create('Ext.data.Store', {
				storeId : 'txt2speech_voices',
				fields : [ 'id', 'name', 'langCode' ],
				proxy : {
					type : 'memory',
					reader : {
						type : 'json',
						root : 'items'
					}
				}
			});
		} else
			Ext.data.StoreManager.lookup('txt2speech_voices').removeAll();

		var sString = dss.ajaxSyncRequest('/json/property/query2', {
			'query' : '/scripts/sonos-controller/media-service/speech/languages/*(code)/voices/*(name,id)'
		});
		result = Ext.JSON.decode(sString);

		for ( var sLangCode in result.result) {
			var landSpecs = result.result[sLangCode];
			Ext.data.StoreManager.lookup('txt2speech_langs').add({
				langCode : sLangCode,
				langName : _(sLangCode)
			});

			for ( var sIndex in landSpecs) {
				var name = landSpecs[sIndex].name;
				var id = landSpecs[sIndex].id;
				Ext.data.StoreManager.lookup('txt2speech_voices').add({
					id : landSpecs[sIndex].id,
					name : landSpecs[sIndex].name,
					langCode : sLangCode
				});
			}
		}
	},
	fillChildsRec : function(array, parentObject) {
		for ( var sKey in array) {
			var oObject = array[sKey];
			if (oObject.parent == parentObject.id) {
				var filesArray = [];
				var newObject = {
					id : oObject.id,
					parent : parentObject,
					files : filesArray,
					child : []
				};
				
				
				if (oObject['name_' + dss.staticDataModel.activeLanguage]!=null)
					newObject.name=oObject['name_' + dss.staticDataModel.activeLanguage];
				else
					newObject.name=oObject.name_en_US;
				
				if (oObject.files != null) {
					for ( var subKey in oObject.files) {
						
						var oObjectFile={
								id : oObject.files[subKey].id,
								url : oObject.files[subKey].url,
								base : newObject
							}
						if (oObject.files[subKey]['name_' + dss.staticDataModel.activeLanguage]!=null)
							oObjectFile.name=oObject.files[subKey]['name_' + dss.staticDataModel.activeLanguage];
						else
							oObjectFile.name=oObject.files[subKey].name_en_US;
						
						
						newObject.files.push(oObjectFile);
					}
				}
				parentObject.child.push(newObject);
				this.fillChildsRec(array, newObject);
			}
		}
	},
	loadMediaLibrary : function() {
		var sString = dss.ajaxSyncRequest('/json/property/query2', {
			'query' : '/scripts/sonos-controller/media-service/library/*(*)/*(*)/*(*)'
		});
		result = Ext.JSON.decode(sString);
		if (result.ok != null)
			if (result.ok == true)
				for ( var sKey in result.result) {
					var oObject = result.result[sKey];
					if (oObject.parent == null) {
						if (oObject.name_de_DE == "Bibliothek") {
							mediaLibrary = {
								id : oObject.id,
								parent : null,
								files : [],
								child : []
							};
							
							if (oObject['name_' + dss.staticDataModel.activeLanguage]!=null)
								mediaLibrary.name=oObject['name_' + dss.staticDataModel.activeLanguage];
							else
								mediaLibrary.name=oObject.name_en_US;
							
							if (oObject.files != null) {
								for ( var subKey in oObject.files) {
									var oObjectFile={
											id : oObject.files[subKey].id,
											url : oObject.files[subKey].url,
											base : mediaLibrary
										};
									if (oObject.files[subKey]['name_' + dss.staticDataModel.activeLanguage]!=null)
										oObjectFile.name=oObject.files[subKey]['name_' + dss.staticDataModel.activeLanguage];
									else
										oObjectFile.name=oObject.files[subKey].name_en_US;
									
									mediaLibrary.files.push(oObjectFile);

								}
							}
							this.fillChildsRec(result.result, mediaLibrary);
						}
					}
				}
	},
	loadFavorites : function() {
		var sString = dss.ajaxSyncRequest('/json/property/query2', {
			'query' : '/scripts/sonos-controller/favorites/*(*)'
		});
		var result = Ext.JSON.decode(sString);
		var arrayData = result.result;

		if (Ext.data.StoreManager.lookup('whattoplay_favorites') != null) {
			Ext.data.StoreManager.lookup('whattoplay_favorites').removeAll();
		} else {
			Ext.create('Ext.data.Store', {
				storeId : 'whattoplay_favorites',
				fields : [ '_id', 'name', 'nameShort', 'next', 'icon', 'listUrl', 'isSingleTrack', 'metaData' ],
				proxy : {
					type : 'memory',
					reader : {
						type : 'json',
						root : 'items'
					}
				}
			});
		}

		var resultArray = [];
		for ( var sKey in arrayData) {
			resultArray.push({
				name : unescape(arrayData[sKey].title),
				_id : (arrayData[sKey].url),
				listUrl : (arrayData[sKey].url),
				metaData : (arrayData[sKey].meta),
				isSingleTrack : arrayData[sKey].singleTrack,
				next : false,
				icon : null
			});
		}

		Ext.data.StoreManager.lookup('whattoplay_favorites').add(resultArray);
	},
	loadPlaylists : function() {
		var sString = dss.ajaxSyncRequest('/json/property/query2', {
			'query' : '/scripts/sonos-controller/playlists/*(*)'
		});
		var result = Ext.JSON.decode(sString);
		var arrayData = result.result;

		if (Ext.data.StoreManager.lookup('whattoplay_playlists') != null) {
			Ext.data.StoreManager.lookup('whattoplay_playlists').removeAll();
		} else {
			Ext.create('Ext.data.Store', {
				storeId : 'whattoplay_playlists',
				fields : [ '_id', 'name', 'next', 'icon', 'listUrl','metaData' ],
				proxy : {
					type : 'memory',
					reader : {
						type : 'json',
						root : 'items'
					}
				}
			});
		}
		var resultArray = [];
		for ( var sKey in arrayData) {
			resultArray.push({
				_id : arrayData[sKey].url,
				name : arrayData[sKey].title,
				next : false,
				icon : null,
				listUrl : arrayData[sKey].url,
				metaData : arrayData[sKey].meta
			});
		}

		Ext.data.StoreManager.lookup('whattoplay_playlists').add(resultArray);
	},
	/* Loading Sonos Playlists, Favorites for every call */
	loadConfigurationData : function(bTestPredefined) {
		return;
		var me = this;
		if ((Date.now() / 1000) - this.lastConfigurationUpdate > 20) { // called
			// from
			// all
			// grids
			// "refresh
			// button",
			// so
			// we
			// limit
			// execution
			// on
			// 10s
			if (bTestPredefined) {
				/*
				 * var sString = dss.ajaxSyncRequest('/json/property/query', { 'query' : '/scripts/sonos-controller/sonos-predifined(activities)' }); var result = Ext.JSON.decode(sString);
				 * oResultObject = dss.parsePropertyQueryAnswer(result, [ { name : 'activities', type : 'simple' } ]); if (oResultObject != null && oResultObject[0] != null &&
				 * oResultObject[0].activities == "3") bUpdateLocalPredefinedActivities = true;
				 */

				// if ip is uknown we get ip of first active sonos
				// speaker
				/*
				 * if (sSonosIP == "") { var sString = dss.ajaxSyncRequest('/json/property/query', { 'query' : '/scripts/sonos-controller/sonos-players/*(*)/*(*)/*(*)/*(*)' }); result =
				 * Ext.JSON.decode(sString); oResultObject = dss.parsePropertyQueryAnswer(result, [ { name : 'ipAddress', type : 'simple' }, { name : 'disabled', type : 'simple' } ]); if
				 * (oResultObject != null) { for (var iIndex = 0; iIndex < oResultObject.length; iIndex++) { if (!JSON.parse(oResultObject[iIndex].disabled) && oResultObject[iIndex].ipAddress != "") {
				 * me.sPlayerIP = oResultObject[iIndex].ipAddress; sSonosIP = me.sPlayerIP; break; } } } }
				 */
			}

			var eventData = new Object();
			eventData.sonosIP = sSonosIP;
			eventData.loadMediaSpeech = (bUpdateLocalPredefinedActivities);

			if (eventData.sonosIP != "" || eventData.loadMediaSpeech) {
				this.lastConfigurationUpdate = (Date.now() / 1000); // remembering
				// last
				// execution
				// time
				return;
			} else {

			}
		}
	},
	whatToPlayRenderer : function(val, metadata, record) {
		var sString = "";

		
		switch (val.type) {
		case 'text2speech':
			sString = _('Text voice: ') + unescape(val.info.text);
			var voiceRecord = Ext.data.StoreManager.lookup('txt2speech_voices').findRecord('id', val.info.voice_id, 0, false, false, true);
			if (voiceRecord != null) {
				var voiceLang = Ext.data.StoreManager.lookup('txt2speech_langs').findRecord('langCode', voiceRecord.get('langCode'), 0, false, false, true);
				sString += ' (' + voiceRecord.get('name') + ' - ' + voiceLang.get('langName') + ')';
			}
			break;
		case 'library':
			var oFileObject = getFileObjectFromMedialib(val.guid);
			sString = oFileObject.name;
			oFileObject = oFileObject.base;
			while (oFileObject.parent != null) {
				sString = oFileObject.name + " -> " + sString;
				oFileObject = oFileObject.parent;
			}
			sString = _("Library: ") + sString;
			break;
		case 'plainRadio':
			var sTitle = _("Radio: ") + val.info.title;
			sString = sTitle;
			break;
		case 'playlist':
			if (val.contentSrc == null)
				sString = '<span style="color:red;">' + _("Playlist: ") + 'invalid selection</span>';
			else {
				var oRecord = Ext.data.StoreManager.lookup('whattoplay_playlists').findRecord('listUrl', val.contentSrc, 0, false, false, true);
				if (oRecord == null)
					sString = '<span style="color:red;">' + _("Playlist: ") + 'invalid selection</span>';
				else
					sString = _("Playlist: ") + oRecord.get('name');
			}
			break;
		case 'favorite':
			if (val.contentSrc == null) {
				sString = '<span style="color:red;">' + _("Favorites: ") + 'invalid selection</span>';
			} else {
				var oRecord = Ext.data.StoreManager.lookup('whattoplay_favorites').findRecord('listUrl', val.contentSrc, 0, false, false, true);
				if (oRecord == null)
					sString = '<span style="color:red;">' + _("Favorites: ") + 'invalid selection</span>';
				else
					sString = _("Favorites: ") + oRecord.get('name');
			}
			break;
		case 'plainSong':
			var sTitle = _("Song: ") + val.info.title;
			if (val.info.artist != null)
				sTitle += ' (' + val.info.artist + ')';
			sString = sTitle;
			break;
		case 'plainQueue':
			var sFirst = true;
			for ( var sKey in val.queue) {
				if (!sFirst)
					sString += ', ';
				sString += val.queue[sKey].title;
				sFirst = false;
			}
			break;
		case 'command':
			switch (val.command) {
			case 'stop':
				sString = _("Command") + ": " + _('Stop');
				break;
			case 'play':
				sString = _("Command") + ": " + _('Play');
				break;
			case 'pause':
				sString = _("Command") + ": " + _('Pause');
				break;
			case 'mute':
				sString = _("Command") + ": " + _('Mute');
				break;
			case 'unmute':
				sString = _("Command") + ": " + _('Unmute');
				break;
			case 'volume_up':
				sString = _("Command") + ": " + _('Volume Up');
				break;
			case 'volume_down':
				sString = _("Command") + ": " + _('Volume Down');
				break;
			default:
				sString = _('Command') + ": " + 'unknown';
				break;
			}
			break;
		}
		if (record.get('predefined') == true)
			return '<span style="color:#aaaaaa;">' + sString + '</span>';
		else
			return sString;

	},
	getContent : function() {
		var me = this;

		me.loadSpeechData();
		me.loadMediaLibrary();
		me.loadFavorites();
		me.loadPlaylists();
		var triggerPreset = Ext.create('DSS.addon.SelectPreset', {});
		var selectRooms = Ext.create('DSS.addon.SelectRooms', {});
		var whatToPlayPreset = Ext.create('DSS.addon.PresetActionSelector', {});
		var whatToPlayActivity = Ext.create('DSS.addon.WhatToPlay', {
			parent : 'Activity'
		});

		me.discover = Ext.create('DSS.component.Editor', {
			title : _('Manually enter IP address'),
			saveTitle : _('Save'),
			staticControls : [ {
				type : 'custom',
				id : 'discoverIP',
				control : Ext.create('DSS.addon.ManualIP', {})
			} ]
		});

		/*
		 * Hiding audio scenes from Activities trigger control
		 */
		var zones = dss.staticDataModel.getZoneArray();
		if (zones != null) {
			for (var iIndex = 0; iIndex < zones.length; iIndex++) {
				var oGroup = dss.staticDataModel.getGroupForZone(zones[iIndex].ZoneID);
				if (oGroup != null) {
					for (var xIndex = 0; xIndex < oGroup.length; xIndex++) {
						if (oGroup[xIndex].group == 4) {
							oGroup[xIndex].reachable = false; // set flag to
							// false so it
							// would not be
							// visible
							// inside
							// control
							break;
						}
					}
				}

				if (Ext.data.StoreManager.lookup('triggerConfigurator_groupStore_' + zones[iIndex].ZoneID) == null) {
					Ext.create('Ext.data.Store', {
						storeId : 'triggerConfigurator_groupStore_' + zones[iIndex].ZoneID,
						fields : [ 'group', 'name', 'icon', 'reachable' ],
						proxy : {
							type : 'memory',
							reader : {
								type : 'json',
								root : 'items'
							}
						}
					});
					Ext.data.StoreManager.lookup('triggerConfigurator_groupStore_' + zones[iIndex].ZoneID).add(oGroup);
				}
			}
		}

		me.discover.width = 580;
		if (me.getUpdateDirectCallback==null) {
			me.getUpdateDirectCallback=function(eventData) {
				if (eventData.actions!='search')
					return;
				if (!fSearchInProgress)
					return;
				fSearchInProgress=false;
				var sMessage = _("Autodiscovery failed!");
				if ((eventData.result != null) && (eventData.result == "true"))
					sMessage = _("Discovery successfully completed.");
				else
					sMessage = _("Discovery failed!");
				Ext.MessageBox.alert(_('Discovery'), sMessage, null);
				me.discoveryDialog.hide();
				me.loadParameter(me.playersGrid);
				return true;
			}
			fChangedCallback.push(this.getUpdateDirectCallback);
		}
		me.discover.on({
			'save' : function(oValue) {
				me.discoveryDialog = new Ext.LoadMask(Ext.getBody(), {
					msg : _('Discovering Sonos devices. Please wait...')
				});
				me.discoveryDialog.show();
				fSearchInProgress=true;
				dss.raiseEvent('sonos.player', {
					actions : 'updatePlayerDirect',
					referenceIP : oValue.discoverIP
				});
			}
		});

		var oConfig = {
			defaultSortCol : 'name',
			disabledProperty : 'disabled',
			inactiveDisabledProperty : 'checkboxShouldBeEnabled',
			storeConfig : {
				storeId : 'pushStore',
				fields : [ {
					name : 'id'
				}, {
					name : 'disabled'
				}, {
					name : 'online'
				}, {
					name : 'typename',
					sortType : 'asUCText'
				}, {
					name : 'deviceName',
					sortType : 'asUCText'
				}, {
					name : 'serial',
					sortType : 'asUCText'
				}, {
					name : 'group',
					sortType : 'asUCText'
				}, {
					name : 'coordinator',
					sortType : 'asUCText'
				}, {
					name : 'zoneID'
				}, {
					name : 'ip'
				} ]
			},
			multiselect : true,
			columns : [ {
				header : _('Active'),
				dataIndex : 'disabled',
				resizable : false,
				xtype : 'disabledcolumn',
				width : 50
			/*
			 * , customColumnRenderer : function(val, metadata, record, rowIdx, colIdx, store, view) { var dispVal = val; if (record.get('online')==false) return ''; return null; }
			 */

			}, {
				header : _('Sonos Room'),
				dataIndex : 'deviceName',
				xtype : 'textcolumn',
				width : 300,
				customColumnRenderer : function(val, metadata, record, rowIdx, colIdx, store, view) {
					var dispVal = val;
					if (record.get('online') == false)
						return '<span style="color:grey;">' + unescape(dispVal) + '</span>';
					return null;
				}
			}, {
				header : _('Players'),
				dataIndex : 'typename',
				xtype : 'textcolumn',
				width : 150,
				customColumnRenderer : function(val, metadata, record, rowIdx, colIdx, store, view) {
					var dispVal = val;
					if (record.get('online') == false)
						return '<span style="color:grey;">' + unescape(dispVal) + '</span>';
					return null;
				}
			}, {
				header : _('Room'),
				dataIndex : 'zoneID',
				xtype : 'textcolumn',
				flex : 1,
				customColumnRenderer : function(val, metadata, record, rowIdx, colIdx, store, view) {
					if (val == null)
						return "-";
					if (val == "")
						return "-";

					var dispVal = dss.staticDataModel.getNameForZone(parseInt("" + val, 10));
					if (record.get('online') == false)
						dispVal = '<span style="color:grey;">' + unescape(dispVal) + '</span>';
					return dispVal;
				}
			}, {
				header : _('Sonos Group'),
				dataIndex : 'dingsda',
				xtype : 'textcolumn',
				flex : 1,
				customColumnRenderer : function(val, metadata, record, rowIdx, colIdx, store, view) {
					var sGroup = record.get('group');
					var iCount = 0;
					var sCoordinator;
					for (var iIndex = 0; iIndex < store.getCount(); iIndex++) {
						var oRecord = store.getAt(iIndex);
						if (oRecord.get('group') == sGroup) {
							iCount++;
							if (oRecord.get('coordinator'))
								sCoordinator = oRecord.get('deviceName');
						}
					}
					if (iCount > 1)
						return sCoordinator;
					else
						return '-';
				}
			}, {
				header : _('Serial Number'),
				dataIndex : 'serial',
				xtype : 'textcolumn',
				width : 150,
				customColumnRenderer : function(val, metadata, record, rowIdx, colIdx, store, view) {
					var dispVal = val;
					if (record.get('online') == false)
						return '<span style="color:grey;">' + unescape(dispVal) + '</span>';
					return null;
				}
			}, {
				header : _('IP Address'),
				dataIndex : 'ip',
				xtype : 'textcolumn',
				width : 150,
				customColumnRenderer : function(val, metadata, record, rowIdx, colIdx, store, view) {
					var dispVal = val;
					if (record.get('online') == false)
						return '<span style="color:grey;">' + unescape(dispVal) + '</span>';
					return null;
				}
			} ],
			buttonText : {
				editButton : _('Assign Room'),
				disableButton : _('Deactivate Player'),
				enableButton : _('Activate Player'),
				deleteButton : _('Delete Player'),
			},
			customButtons : [ {
				text : _('Autodiscovery'),
				iconCls : 'icon-discovery',
				handler : function(grid, entry) {
					me.discoveryDialog = new Ext.LoadMask(Ext.getBody(), {
						msg : _('Auto discovering Sonos devices. Please wait...')
					});
					me.discoveryDialog.show();
					var sNetworkIP = "";
					var sResponseIP = dss.ajaxSyncRequest('/cgi-bin/netinfo.sh', {});
					if (sResponseIP != "") {
						var dssNet = JSON.parse(sResponseIP);
						if (dssNet != null && dssNet.data != null & dssNet.data.ip != null)
							sNetworkIP = dssNet.data.ip;
						
						if (me.getUpdateAutoCallback==null) {
							me.getUpdateAutoCallback=function(eventData) {
								if (eventData.actions!='search')
									return;
								if (!fSearchInProgress)
									return;
								fSearchInProgress=false;
								var sMessage = _("Autodiscovery failed!");
								if ((eventData.result != null) && (eventData.result == "true"))
									sMessage = _("Autodiscovery successfully completed.");
								else
									sMessage = _("Autodiscovery completed. No Sonos devices found.");
								Ext.MessageBox.alert(_('Autodiscovery info'), sMessage, null);
								me.discoveryDialog.hide();
								me.loadParameter(grid);
								return true;
							}
							fChangedCallback.push(this.getUpdateAutoCallback);
						}
						
						fSearchInProgress=true;
						dss.raiseEvent('sonos.player', {
							actions : 'startAutoDiscovery',
							ip : sNetworkIP
						});

					} else {
						me.discoveryDialog.hide();
						Ext.MessageBox.alert(_('Autodiscovery info'), _('No network info available for Autodiscovery, please use Manual enter IP address button.'), null);
					}

				}
			}, {
				text : _('Manually enter IP address'),
				iconCls : 'icon-manual-discovery',
				handler : function(grid, entry) {
					me.discover.setValues({
						discoverIP : ""
					});
					me.discover.show();
				}
			}

			],
			editorConfig : {
				title : _('Player Info'),
				staticControls : [ {
					type : 'hidden',
					id : 'id'
				}, {
					type : 'hidden',
					id : 'online'
				}, {
					type : 'custom',
					id : 'deviceName',
					control : Ext.create('Ext.form.field.Display', {
						fieldLabel : _('Sonos Room'),
						height : 20
					})
				}, {
					type : 'custom',
					id : 'typename',
					control : Ext.create('Ext.form.field.Display', {
						fieldLabel : _('Devices'),
						height : 20
					})
				}, {
					type : 'custom',
					id : 'zoneID',
					control : Ext.create('DSS.component.SonosDeviceLabel', {})
				}, ],
			},
			listeners : {
				'loadDataStore' : function(grid) {
					me.loadParameter(grid);
					me.playersGrid = grid;
					dss.raiseEvent('sonos.player',{actions:'updatePlayer'});
				},
				'saveEntry' : function(grid, values) {
					me.waitAndReloadZone(grid, {
						actions : 'setZone',
						zoneID : values.zoneID,
						id : values.id
					});
				},
				'deleteEntry' : function(grid, entry) {
					var idVal;
					if (entry.get != null) {
						idVal = [ entry.get('id') ];
					} else {
						idVal = [];
						for (var iIndex = 0; iIndex < entry.length; iIndex++)
							if (entry[iIndex] != null)
								idVal.push(entry[iIndex].get('id'));
					}
					me.waitAndReloadZone(grid, {
						actions : 'resetZone',
						id : Ext.JSON.encode(idVal)
					});
				},
				'disableEntry' : function(grid, entry) {
					var idVal;
					if (entry.get != null) {
						idVal = [ entry.get('id') ];
					} else {
						idVal = [];
						for (var iIndex = 0; iIndex < entry.length; iIndex++)
							if (entry[iIndex] != null)
								idVal.push(entry[iIndex].get('id'));
					}
					me.waitAndReloadZone(grid, {
						actions : 'disable',
						id : Ext.JSON.encode(idVal)
					});
				},
				'selectionChanged' : function(grid, entry) {
				},
				'enableEntry' : function(grid, entry) {
					var idVal;
					if (entry.get != null) {
						idVal = entry.get('id');
					} else {
						idVal = [];
						for (var iIndex = 0; iIndex < entry.length; iIndex++)
							if (entry[iIndex] != null)
								idVal.push(entry[iIndex].get('id'));
					}
					me.waitAndReloadZone(grid, {
						actions : 'enable',
						id : Ext.JSON.encode(idVal)
					});
				},
			}
		};

		var oConfigPR = {
			inactiveField : 'predefined',
			grouperField : 'grouping',
			defaultSortCol : 'predefined',
			disabledProperty : 'disabled',
			inactiveDisabledProperty : 'predefined',
			storeConfig : {
				storeId : 'pushStorePr',
				fields : [ {
					name : 'id'
				}, {
					name : 'predefined'
				}, {
					name : 'disabled',
					sortType : 'asBoolean'
				}, {
					name : 'triggers',
					sortType : 'asTrigger'
				}, {
					name : 'action',
					sortType:'asSonosAction'
				}, {
					name : 'grouping'
				}, {
					name : 'zoneID'
				}, {
					name : 'audioName'
				} ]
			},
			multiselect : true,
			columns : [ {
				header : _('Active'),
				dataIndex : 'disabled',
				resizable : false,
				xtype : 'disabledcolumn',
				width : 50
			}, {
				header : _('Name'),
				dataIndex : 'triggers',
				xtype : 'textcolumn',
				flex : 3,
				customColumnRenderer : function(val, metadata, record) {
					var sString = dss.staticDataModel.getNameForScene(val[0].zone, val[0].group, val[0].scene);

					if (record.get('predefined') == true)
						return '<span style="color:#aaaaaa;">' + sString + '</span>';
					else
						return sString;

				}
			}, {
				header : _('Trigger Preset'),
				dataIndex : 'triggers',
				xtype : 'textcolumn',
				flex : 3,
				customColumnRenderer : function(val, metadata, record) {

					var sString = '<table><tr><td style="padding-right:5px;">' + dss.staticDataModel.getImageForScene(val[0].zone, val[0].group, val[0].scene) + "</td>";
					if (record.get('predefined') == true)
						sString += '<td style="color:#aaaaaa;">';
					else
						sString += '<td style="">';
					var sTexte = [];
					for (var iIndex = 0; iIndex < val.length; iIndex++) {
						if (sTexte.indexOf(dss.staticDataModel.getDefaultNameForScene(val[iIndex].zone, val[iIndex].group, val[iIndex].scene)) == -1)
							sTexte.push(dss.staticDataModel.getDefaultNameForScene(val[iIndex].zone, val[iIndex].group, val[iIndex].scene));
					}
					for (var iIndex = 0; iIndex < sTexte.length; iIndex++) {
						if (iIndex > 0)
							sString += ", "
						sString += sTexte[iIndex];
					}
					sString += '</td></tr></table>';
					return sString;
				}
			}, {
				header : _('What To Play'),
				dataIndex : 'action',
				xtype : 'textcolumn',
				flex : 3,
				customColumnRenderer : this.whatToPlayRenderer
			}, {
				header : _('Volume'),
				dataIndex : 'action',
				xtype : 'textcolumn',
				width : 75,
				customColumnRenderer : function(val, metadata, record, rowIdx, colIdx, store, view) {
					var sString = "-";
					if (val.volume != null)
						sString = (val.volume + ' %');

					if (record.get('predefined') == true)
						return '<span style="color:#aaaaaa;">' + sString + '</span>';
					else
						return sString;
				}
			} ],
			buttonText : {
				newButton : _('New Preset'),
				editButton : _('Edit Preset'),
				deleteButton : _('Delete Preset'),
				disableButton : _('Deactivate Preset'),
				enableButton : _('Activate Preset'),
				testButton : _('Test Preset'),
			},
			editorConfig : {
				title : _('Manage Room Preset'),
				defaultTabOnEdit : 'whatToPlay',
				staticControls : [ {
					type : 'hidden',
					id : 'id'
				}, {
					type : 'text',
					label : _('Action name'),
					id : 'audioName'
				} ],
				tabControls : [ {
					tabLabel : _('Trigger'),
					tabDescription : _('Select the room and preset that triggers the music/command.'),
					content : [ {
						type : 'custom',
						id : 'triggers',
						control : triggerPreset
					} ]
				}, {
					tabLabel : _('What To Play'),
					tabDescription : _('Select the music that will be played/executed.'),
					content : [ {
						type : 'custom',
						id : 'action',
						control : whatToPlayPreset
					} ]
				} ],
			},
			listeners : {
				'loadDataStore' : function(grid) {
					me.loadPresets(grid);
					me.presetsGrid = grid;
					dss.raiseEvent('sonos.info',{actions:'fetchSonosSettings'});
				},
				'saveEntry' : function(grid, values) {
					var sAudioSceneName = unescape(values.audioName);
					values.target = {
						singularyZone : [ values.triggers[0].zone ]
					};
					me.waitAndReloadPreset(grid, {
						actions : 'save',
						value : Ext.JSON.encode(values)
					});
					Ext.Ajax.request({
						url : "/json/zone/sceneSetName?id=" + values.triggers[0].zone + "&groupID=4&sceneNumber=" + values.triggers[0].scene + "&newName=" + sAudioSceneName
					});
				},
				'deleteEntry' : function(grid, entry) {
					var idVal;
					if (entry.get != null) {
						idVal = entry.get('id');
						// deleting audio scene name
						Ext.Ajax.request({
							url : "/json/zone/sceneSetName?id=" + entry.get("roomID") + "&groupID=4&sceneNumber=" + entry.get("presetID") + "&newName="
						});
					} else {
						idVal = [];
						for (var iIndex = 0; iIndex < entry.length; iIndex++)
							if (entry[iIndex] != null) {
								idVal.push(entry[iIndex].get('id'));
								Ext.Ajax.request({
									url : "/json/zone/sceneSetName?id=" + entry[iIndex].get("roomID") + "&groupID=4&sceneNumber=" + entry[iIndex].get("presetID") + "&newName="
								});
							}
					}

					me.waitAndReloadPreset(grid, {
						actions : 'delete',
						id : Ext.JSON.encode(idVal)
					});
				},
				'disableEntry' : function(grid, entry) {
					var idVal;
					if (entry.get != null) {
						idVal = entry.get('id');
					} else {
						idVal = [];
						for (var iIndex = 0; iIndex < entry.length; iIndex++)
							if (entry[iIndex] != null)
								idVal.push(entry[iIndex].get('id'));
					}

					me.waitAndReloadPreset(grid, {
						actions : 'disable',
						id : Ext.JSON.encode(idVal)
					});
				},
				'enableEntry' : function(grid, entry) {
					var idVal;
					if (entry.get != null) {
						idVal = entry.get('id');
					} else {
						idVal = [];
						for (var iIndex = 0; iIndex < entry.length; iIndex++)
							if (entry[iIndex] != null)
								idVal.push(entry[iIndex].get('id'));
					}

					me.waitAndReloadPreset(grid, {
						actions : 'enable',
						id : Ext.JSON.encode(idVal)
					});
				},
				'testEntry' : function(grid, entry) {
					var infoDialog = new Ext.LoadMask(Ext.getBody(), {
						msg : _('Testing preset in progress. Please wait ... ')
					});
					infoDialog.show();

					var activityPath = '/scripts/sonos-controller/entries/' + entry.get('id');
					
					if (me.callbackTestEntry==null) {
						me.callbackTestEntry=function (oData) {
							if (oData.actions=='execute') {								
								infoDialog.hide();
							}
						}
						fChangedCallback.push(me.callbackTestEntry);
					}
					dss.raiseEvent('sonos.execute',{path:activityPath});

				}
			}
		};

		var oConfigAct = {
			defaultSortCol : 'roomName',
			disabledProperty : 'disabled',
			inactiveDisabledProperty : 'predefined',
			storeConfig : {
				storeId : 'pushStoreAct',
				fields : [ {
					name : 'id'
				}, {
					name : 'disabled',
					sortType : 'asBoolean'
				}, {
					name : 'triggers',
					sortType : 'asTrigger'
				}, {
					name : 'action',
					sortType:'asSonosAction'
				}, {
					name : 'target'
				}, {
					name : 'conditions'
				}, {
					name : 'implicitRestore'
				}, {
					name: 'metaData'
				} ]
			},
			multiselect : true,
			columns : [ {
				header : _('Active'),
				dataIndex : 'disabled',
				resizable : false,
				xtype : 'disabledcolumn',
				width : 50
			}, {
				header : _('Trigger'),
				dataIndex : 'triggers',
				xtype : 'triggercolumn',
				flex : 3
			}, {
				header : _('What To Play'),
				dataIndex : 'action',
				xtype : 'textcolumn',
				flex : 3,
				customColumnRenderer : this.whatToPlayRenderer
			}, {
				header : _('Where To Play'),
				dataIndex : 'target',
				xtype : 'textcolumn',
				flex : 3,
				customColumnRenderer : function(val, metadata, record, rowIdx, colIdx, store, view) {
					var oArray;
					if (val.singularyZone != null)
						oArray = val.singularyZone;
					else if (val.grouped != null)
						oArray = val.grouped;
					else
						return _('<span style="color:red;">invalid selection</span>');
					if (oArray.length == 1)
						if (oArray[0] == 0) {
							if (val.grouped != null)
								return _("All rooms") + ' *';
							else
								return _("All rooms");
						}

					var fFirst = false;
					var sRooms = "";
					for (var yIndex = 0; yIndex < oArray.length; yIndex++) {
						if (yIndex > 0)
							sRooms += ", ";
						sRooms += dss.staticDataModel.getNameForZone(oArray[yIndex])
					}
					if (val.grouped != null)
						sRooms += '*';
					return sRooms;
				}
			}, {
				header : _('Volume'),
				dataIndex : 'action',
				xtype : 'textcolumn',
				width : 75,
				customColumnRenderer : function(val, metadata, record, rowIdx, colIdx, store, view) {
					if (val.volume != null)
						return (val.volume + ' %');
					else
						return '-';
				}
			} ],
			buttonText : {
				newButton : _('New Activity'),
				editButton : _('Edit Activity'),
				deleteButton : _('Delete Activity'),
				disableButton : _('Deactivate Activity'),
				enableButton : _('Activate Activity'),
				testButton : _('Test Activity'),
			},
			editorConfig : { 
				title : _('Manage Activity'),
				staticControls : [ {
					type : 'hidden',
					id : 'id'
				},{
					type : 'hidden',
					id : 'implicitRestore'
				},{
					type : 'hidden',
					id : 'metaData'
				}  ],
				tabControls : [ {
					tabLabel : _('Trigger'),
					defaultTabOnNew : true,
					tabDescription : _('Choose the triggering activity for the message. You can choose more than one activity as trigger.'),
					content : [ {
						type : 'TriggerConfigurator',
						id : 'triggers',
						flex : true
					} ]
				}, {
					tabLabel : _('What To Play'),
					defaultTabOnEdit : true,
					tabDescription : _('Select the music/command that will be played/executed.'),
					content : [ {
						type : 'custom',
						id : 'action',
						control : whatToPlayActivity
					} ]
				}, {
					tabLabel : _('Where To Play'),
					tabDescription : _('Select the rooms where music should be played.'),
					content : [ {
						type : 'custom',
						id : 'target',
						control : selectRooms
					} ]
				} ],
			},
			listeners : {
				'loadDataStore' : function(grid) {
					me.loadEntries(grid);
					me.activitiesGrid = grid;
					dss.raiseEvent('sonos.info',{actions:'fetchSonosSettings'});
				},
				'saveEntry' : function(grid, values) {
					var fShouldAddImplicit=false;
					for (var iIndex=0;iIndex<values.triggers.length;iIndex++) {
						if (values.triggers[iIndex].type=='state-change') {
							if (values.triggers[iIndex].name=='fire')
								fShouldAddImplicit=true;
						}
						if ((values.triggers[iIndex].type=='zone-scene')||(values.triggers[iIndex].type=='bus-zone-scene')) {
							if (values.triggers[iIndex].zone==0)
								if (values.triggers[iIndex].group==0) {
									if (values.triggers[iIndex].scene==74)
										fShouldAddImplicit=true;
									if (values.triggers[iIndex].scene==65)
										fShouldAddImplicit=true;
									if (values.triggers[iIndex].scene==83)
										fShouldAddImplicit=true;
									if (values.triggers[iIndex].scene==84)
										fShouldAddImplicit=true;
									if (values.triggers[iIndex].scene==85)
										fShouldAddImplicit=true;
								}
						}
					}
					if (fShouldAddImplicit)
						values.implicitRestore=true;
					me.waitAndReloadActivities(grid, {
						actions : 'save',
						value : Ext.JSON.encode(values)
					});
				},
				'deleteEntry' : function(grid, entry) {
					var idVal;
					if (entry.get != null) {
						idVal = entry.get('id');
					} else {
						idVal = [];
						for (var iIndex = 0; iIndex < entry.length; iIndex++)
							if (entry[iIndex] != null)
								idVal.push(entry[iIndex].get('id'));
					}
					me.waitAndReloadActivities(grid, {
						actions : 'delete',
						id : Ext.JSON.encode(idVal)
					});
				},
				'disableEntry' : function(grid, entry) {
					var idVal;
					if (entry.get != null) {
						idVal = entry.get('id');
					} else {
						idVal = [];
						for (var iIndex = 0; iIndex < entry.length; iIndex++)
							if (entry[iIndex] != null)
								idVal.push(entry[iIndex].get('id'));
					}

					me.waitAndReloadActivities(grid, {
						actions : 'disable',
						id : Ext.JSON.encode(idVal)
					});
				},
				'enableEntry' : function(grid, entry) {
					var idVal;
					if (entry.get != null) {
						idVal = entry.get('id');
					} else {
						idVal = [];
						for (var iIndex = 0; iIndex < entry.length; iIndex++)
							if (entry[iIndex] != null)
								idVal.push(entry[iIndex].get('id'));
					}
					me.waitAndReloadActivities(grid, {
						actions : 'enable',
						id : Ext.JSON.encode(idVal)
					});
				},
				'testEntry' : function(grid, entry) {
					var infoDialog = new Ext.LoadMask(Ext.getBody(), {
						msg : _('Testing activity in progress. Please wait ... ')
					});
					infoDialog.show();

					var activityPath = '/scripts/sonos-controller/entries/' + entry.get('id');

					if (me.callbackTestEntry==null) {
						me.callbackTestEntry=function (oData) {
							if (oData.actions=='execute') {								
								infoDialog.hide();
							}
						}
						fChangedCallback.push(me.callbackTestEntry);
					}
					dss.raiseEvent('sonos.execute',{path:activityPath});

				}

			}
		};

		var oMainGrid = Ext.create('DSS.gridPanel', oConfig);
		me.oPlayers = Ext.create('Ext.panel.Panel', {
			title : _('Players'),
			layout : 'fit',
			items : [ oMainGrid ]
		});
		var oMainPresets = Ext.create('DSS.gridPanel', oConfigPR);
		me.oPresets = Ext.create('Ext.panel.Panel', {
			title : _('Room Presets'),
			layout : 'fit',
			items : [ oMainPresets ]
		});
		var oMainActivities = Ext.create('DSS.gridPanel', oConfigAct);
		me.oActivities = Ext.create('Ext.panel.Panel', {
			title : _('Global Activities'),
			layout : 'fit',
			items : [ oMainActivities ]
		});
		oMainGrid.editorWindow.setWidth(420);
		me.oMainGrid = oMainGrid;
		oMainGrid.addListener('selectionChange', function(view, selections, options) {
			if (selections.length > 0) {
				var fEnabled = false;
				var fDisabled = false;
				for (var iIndex = 0; iIndex < selections.length; iIndex++) {
					fEnabled = fEnabled || (selections[iIndex].get('online'));
					fDisabled = fDisabled || (!selections[iIndex].get('online'));
				}
				if (fEnabled && fDisabled) {
					for (var iIndex = 0; iIndex < selections.length; iIndex++) {
						if (!selections[iIndex].get('online'))
							view.deselect(selections[iIndex]);
					}
				} else {
					me.oMainGrid;
					if (fEnabled) {
						me.oMainGrid.btnDelete.setDisabled(true);
					}
					if (fDisabled) {
						me.oMainGrid.btnEnable.setDisabled(true);
						me.oMainGrid.btnEdit.setDisabled(true);
						me.oMainGrid.btnDelete.setDisabled(false);
					}
				}
			}
		}, oMainGrid);

		// Catching cancel button on edit dialog for activities
		var oEditorAct = oMainActivities.editorWindow;

		oMainActivities.editorWindow.on({
			'show' : function(oValue) {
				me.lastConfigurationUpdate = 0;
				me.loadConfigurationData("", false);
			},

		});

		oMainActivities.editorWindow.buttonCancel.setHandler(function() {
			var infoDialog = oEditorAct.editControls.action.infoDialog;
			if (infoDialog != null)
				infoDialog.hide();
			if (oEditorAct.editControls.action.testPlayStopSubscriptID != null && oEditorAct.editControls.action.testPlayStopSubscriptID > 0) {

				dss.ajaxAssyncRequestWithoutAnswer('/json/event/unsubscribe', {
					subscriptionID : oEditorAct.editControls.action.testPlayStopSubscriptID,
					name : 'sonos-controller-test-play-stopped'
				});
			}

			var t2s = oEditorAct.editControls.action.t2sCreatedObj;
			if (t2s != null) {
				// we must delete created text2speech files that are not
				// saved and used
				if (t2s.guid != "" && t2s.guid != t2s.originalGuid) {
					dss.raiseEvent('sonos-controller.testPlay', {
						actions : 'delete',
						value : Ext.JSON.encode(t2s.guid)
					});
				}
			}

			oEditorAct.fireEvent('cancel');
			oEditorAct.hide();
		});

		me.mainAddonPanel = Ext.create('Ext.tab.Panel', {
			layout : 'fit',
			cls : 'generalTab',
			items : [ me.oPresets, me.oActivities, me.oPlayers ]
		});
		me.setupTabs();

		var oRecords = []
		for (var iIndex = 0; iIndex < Ext.data.StoreManager.lookup('triggerConfigurator_securityStore').getCount(); iIndex++) {
			var oRecord = Ext.data.StoreManager.lookup('triggerConfigurator_securityStore').getAt(iIndex);
			var _id = oRecord.get('_id');
			if (parseInt(_id, 10) > 1000)
				oRecords.push(oRecord);
			if (_id == 'fire_inactive')
				oRecords.push(oRecord);
		}
		Ext.data.StoreManager.lookup('triggerConfigurator_securityStore').remove(oRecords);

		
		
		if (this.callbackWaitAndReloadZone==null) {
			
			this.callbackWaitAndReloadZone=function (oData) {
				if ((oData.actions=='ensurePredefinedEntries') ||
					(oData.actions=='enablePlayer') ||
					(oData.actions=='structureUpdate') ||
					(oData.actions=='disablePlayer') ||
					(oData.actions=='resetZone')) {
					
					me.loadParameter(me.oMainGrid);
					if (me.presetsGrid != null) {
						me.loadPresets(me.presetsGrid);
					}
					if (me.activitiesGrid != null) {
						me.loadEntries(me.activitiesGrid);
					}
				}
			}
			fChangedCallback.push(this.callbackWaitAndReloadZone);
			
			this.callbackWaitAndReloadPresets=function (oData) {
				if ((oData.actions=='config') ||
					(oData.actions=='enable') ||
					(oData.actions=='disable') ||
					(oData.actions=='delete')) {
					
					dss.staticDataModel.initNameCache();
					dss.staticDataModel.initSpecialActions();
					dss.staticDataModel.initDeviceCache();
					
					me.loadPresets(me.presetsGrid);
					me.loadEntries(me.activitiesGrid); 
					me.waitDialog.hide(); 
				}
			}
			fChangedCallback.push(this.callbackWaitAndReloadPresets);

			this.callbackErrorMessages=function (oData) {
				if (oData.actions=='text2speech') {					
					me.waitDialog.hide(); 
					Ext.MessageBox.alert(_('Error creating Text To Speech file'), _('Error creating Text To Speech file. File was not created.'), null);					
				}
			}
			fChangedCallback.push(this.callbackErrorMessages);

			this.callbackSonosSettings=function (oData) {
				if (oData.actions=='sonosSettings') {
					me.loadFavorites();
					me.loadPlaylists();
				}
			}
			fChangedCallback.push(this.callbackSonosSettings);
			
		}
		
		return me.mainAddonPanel;
	},

	setupTabs : function() {
		var me = this;
		if (me.mainAddonPanel != null) {
			me.oPresets.setDisabled(false);
			me.oActivities.setDisabled(false);
			if (me.bDisableTabs) {
				me.oPresets.setDisabled(true);
				me.oActivities.setDisabled(true);
				me.mainAddonPanel.setActiveTab(me.oPlayers);
			}
		}
	},

	loadParameter : function(grid) {
		grid.setStatus(_('Loading ...'));
		grid.setDisabled(true);
		var me = this;
		var sString = dss.ajaxSyncRequest('/json/property/query', {
			'query' : '/scripts/sonos-controller/players/*(*)'
		});
		var result = Ext.JSON.decode(sString);

		if (Ext.data.StoreManager.lookup('whattoselect_rooms') == null) {
			Ext.create('Ext.data.Store', {
				storeId : 'whattoselect_rooms',
				fields : [ 'roomID', 'roomName', 'passive' ],
				proxy : {
					type : 'memory',
					reader : {
						type : 'json',
						root : 'items'
					}
				},
				sorters : [ {
					property : 'roomName',
					direction : 'ASC'
				} ]
			});
		} else
			Ext.data.StoreManager.lookup('whattoselect_rooms').removeAll();

		if (Ext.data.StoreManager.lookup('whattoplay_rooms') == null) {
			Ext.create('Ext.data.Store', {
				storeId : 'whattoplay_rooms',
				fields : [ '_id', 'name' ],
				proxy : {
					type : 'memory',
					reader : {
						type : 'json',
						root : 'items'
					}
				}
			});
		} else
			Ext.data.StoreManager.lookup('whattoplay_rooms').removeAll();

		if (Ext.data.StoreManager.lookup('selectpreset_l1') == null) {
			Ext.create('Ext.data.Store', {
				storeId : 'selectpreset_l1',
				fields : [ '_id', 'name', 'next', 'icon', 'passive' ],
				proxy : {
					type : 'memory',
					reader : {
						type : 'json',
						root : 'items'
					}
				}
			});
		} else
			Ext.data.StoreManager.lookup('selectpreset_l1').removeAll();

		this.bDisableTabs = true;
		this.sPlayerIP = "";

		var oRoomArray = [];
		var oAndererRoomArray = [];
		var oArray = [];
		this.filterZone = [];
		if (result.result != null) {
			for ( var sKey in result.result.players) {
				var oDataObject = result.result.players[sKey];
				var oPushObject = {
					id : oDataObject.id,
					disabled : oDataObject.disabled,
					online : oDataObject.online,
					typename : oDataObject.typename,
					deviceName : oDataObject.deviceName,
					serial : oDataObject.serial,
					group : oDataObject.group,
					coordinator : oDataObject.coordinator,
					zoneID : oDataObject.zoneID,
					ip : oDataObject.ip,
					invisible : oDataObject.invisible
				}

				if (oDataObject.zoneID != null) 
					if (this.filterZone.indexOf(parseInt(oDataObject.zoneID, 10)) == -1)
						this.filterZone.push(parseInt(oDataObject.zoneID, 10));
				if (oDataObject.zoneID != null)
					if (!oPushObject.invisible)
						/*if (oPushObject.online) */{
							oRoomArray.push(oDataObject.zoneID);
							this.bDisableTabs = false;
							Ext.data.StoreManager.lookup('whattoplay_rooms').add({
								_id : oDataObject.zoneID,
								name : dss.staticDataModel.getNameForZone(oDataObject.zoneID),
							});
							Ext.data.StoreManager.lookup('selectpreset_l1').add({
								_id : oDataObject.zoneID,
								name : dss.staticDataModel.getNameForZone(oDataObject.zoneID),
								icon : null,
								passive : false,
								next : true
							});
							Ext.data.StoreManager.lookup('whattoselect_rooms').add({
								roomID : oDataObject.zoneID,
								roomName : dss.staticDataModel.getNameForZone(oDataObject.zoneID),
								passive : false
							});
						}
				if (!oPushObject.invisible)
					oArray.push(oPushObject);
			}

			// reloading sonos playlists
			grid.loadParameter(oArray);

			playerStore=grid.store;
			if (this.presetsGrid!=null) {
				this.presetsGrid.store.clearFilter(true);
				this.presetsGrid.store.filterBy(function(record) {
					return (me.filterZone.indexOf(record.get('zoneID')) != -1);
				});
			}
			/*
			 * if (me.sPlayerIP != "") { if (me.sPlayerIP.indexOf(", ") != -1) { // if more // than one // IP, we // remember // first one var tmpData = me.sPlayerIP.split(", "); if (tmpData != null &&
			 * tmpData.length > 0) me.sPlayerIP = tmpData[0]; } } me.loadConfigurationData(me.sPlayerIP, true);
			 */
		}

		this.setupTabs();
		this.loadPlaylists();
		this.loadFavorites();
		this.loadMediaLibrary();
		this.loadSpeechData();

		grid.setStatus(_('Ready'));
		grid.setDisabled(false);
	},
	prepareWhatTopPlayInfo : function(oResultObject, sSubPath) {
		var sPlayInfo = "";
		var nType = (parseInt("" + oResultObject.actionType, 10));
		switch (nType) {
		case 0:
			var nowAction = parseInt("" + oResultObject.nowAction, 10);
			if (nowAction == 0) { // single song/radio
				var isRadio = JSON.parse(oResultObject.nowSingleIsRadio);
				sPlayInfo = _("Song: ");
				if (isRadio)
					sPlayInfo = _("Radio: ");
				if (oResultObject.nowSingleTitle != "")
					sPlayInfo += oResultObject.nowSingleTitle;
				if (oResultObject.nowSingleArtist != "")
					sPlayInfo += " (" + oResultObject.nowSingleArtist + ")";
			} else { // active song queue
				sString = dss.ajaxSyncRequest('/json/property/query', {
					'query' : '/scripts/sonos-controller/' + sSubPath + '/' + oResultObject.id + '/active-queue/*(*)/*(*)/*(*)/*(*)'
				});
				var result = Ext.JSON.decode(sString);
				var arrayQueue = dss.parsePropertyQueryAnswer(result, [ {
					name : 'id',
					type : 'simple'
				}, {
					name : 'title',
					type : 'simple'
				}, {
					name : 'artist',
					type : 'simple'
				} ]);
				if (arrayQueue != null && arrayQueue.length > 0) {
					var sTitle = "";
					for (i = 0; i < arrayQueue.length; i++) {
						if (i > 2) {
							sTitle += "...";
							break;
						}

						if (i > 0)
							sTitle += ", ";
						sTitle += arrayQueue[i].title;
					}
					sPlayInfo = sTitle;
				}
			}
			break;

		case 1: // playlist
			sPlayInfo = _("Playlist: ") + unescape(oResultObject.playListTitle);
			break;

		case 2: // favorites
			sPlayInfo = _("Favorites: ") + unescape(oResultObject.favoritesTitle);
			break;

		case 3: // Media library
			sPlayInfo = _("Library: ") + unescape(oResultObject.mediaFileTitle);
			break;

		case 4:
			sPlayInfo = _("Text voice: ") + unescape(oResultObject.t2Stext);
			break;

		case 5: // command
			var sCommand = "";
			switch (parseInt("" + oResultObject.command, 10)) {
			case 1:
				sCommand = _('Stop');
				break;
			case 2:
				sCommand = _('Play');
				break;
			case 3:
				sCommand = _('Pause');
				break;
			case 4:
				sCommand = _('Mute');
				break;
			case 5:
				sCommand = _('Unmute');
				break;
			case 6:
				sCommand = _('Volume Up');
				break;
			case 7:
				sCommand = _('Volume Down');
				break;

			}
			sPlayInfo = _("Command") + ": " + sCommand;
			break;
		}
		return sPlayInfo;
	},
	loadPresets : function(grid) {
		grid.setStatus(_('Loading ...'));
		grid.setDisabled(true);

		this.loadConfigurationData(true);

		dss.staticDataModel.initHLECache();

		var me = this;
		sString = dss.ajaxSyncRequest('/json/property/query2', {
			'query' : '/scripts/sonos-controller/entries/*(*)/*(*)/*(*)/*(*)'
		});
		result = Ext.JSON.decode(sString);
		var gridArray = [];
		if (result.result != null) {
			for ( var sKey in result.result) {
				var gridObject = {};
				var oResultObject = result.result[sKey];

				gridObject.id = oResultObject.id;

				gridObject.conditions = oResultObject.conditions;

				gridObject.disabled = false;
				if (oResultObject.conditions != null)
					if (oResultObject.conditions.enabled != null)
						gridObject.disabled = !oResultObject.conditions.enabled;

				gridObject.triggers = [];
				for ( var oKey in oResultObject.triggers)
					gridObject.triggers.push(oResultObject.triggers[oKey]);
				for (var iIndex = 0; iIndex < gridObject.triggers.length; iIndex++) {
					if (gridObject.triggers[iIndex].type == 'bus-zone-scene')
						gridObject.triggers[iIndex].type = 'zone-scene';
				}

				gridObject.audioName = dss.unescapeProperty(dss.staticDataModel.getNameForScene(gridObject.triggers[0].zone, gridObject.triggers[0].group, gridObject.triggers[0].scene));
				gridObject.action = oResultObject.action;
				if (gridObject.action.info!=null)
					for (var sKey in gridObject.action.info) 
						gridObject.action.info[sKey]=dss.unescapeProperty(gridObject.action.info[sKey]);
				
				if (gridObject.action.contentSrc!=null)
					gridObject.action.contentSrc=dss.unescapeProperty(gridObject.action.contentSrc);

				if (oResultObject.target != null)
					if (oResultObject.target.singularyZone != null) {
						gridObject.target = {
							singularyZone : []
						};
						for ( var sKey in oResultObject.target.singularyZone)
							gridObject.target.singularyZone.push(oResultObject.target.singularyZone[sKey]);
					} else {
						gridObject.target = {
							grouped : []
						};
						for ( var sKey in oResultObject.target.grouped)
							gridObject.target.grouped.push(oResultObject.target.grouped[sKey]);
					}

				if (gridObject.target.singularyZone != null)
					if (gridObject.target.singularyZone.length == 1)
						if (gridObject.triggers.length > 0)
							if (gridObject.triggers[0].type == 'zone-scene')
								if (gridObject.triggers[0].group == 4)
									if (gridObject.triggers[0].zone == gridObject.target.singularyZone[0]) 
										if (this.filterZone.indexOf(gridObject.triggers[0].zone)!=-1) {
											gridObject.zoneID = gridObject.target.singularyZone[0];
											gridObject.grouping = dss.staticDataModel.getNameForZone(gridObject.target.singularyZone[0]);
											gridArray.push(gridObject);
										}
				if (oResultObject.metaData != null)
					if (oResultObject.metaData!='') {
						var oObject = JSON.parse(dss.unescapeProperty(oResultObject.metaData));
						if (oObject.type != null) {
							if (oObject.type == 'predefinedStopEntry')
								gridObject.predefined = true;
							if (oObject.type == 'predefinedStartEntry')
								gridObject.predefined = true;
					}
				}

			}
			grid.loadParameter(gridArray);
		}
		presetStore=grid.store;

		grid.setStatus(_('Ready'));
		grid.setDisabled(false);
	},
	waitAndReloadZone : function(grid, parameter) {
		grid.setStatus(_('Saving ...'));
		grid.setDisabled(true);
		dss.raiseEvent('sonos.player', parameter);
	},
	waitAndReloadPreset : function(grid, parameter) {
		grid.setStatus(_('Saving ...'));
		grid.setDisabled(true);
		var me = this;
		dss.raiseEvent('sonos.config', parameter);		
	},
	loadEntries : function(grid) {
		grid.setStatus(_('Loading ...'));
		grid.setDisabled(true);
		this.loadConfigurationData(true);
		var me = this;
		sString = dss.ajaxSyncRequest('/json/property/query2', {
			'query' : '/scripts/sonos-controller/entries/*(*)/*(*)/*(*)/*(*)'
		});
		result = Ext.JSON.decode(sString);
		var gridArray = [];
		if (result.result != null) {
			for ( var sKey in result.result) {
				var gridObject = {};
				var oResultObject = result.result[sKey];

				gridObject.id = oResultObject.id;

				gridObject.conditions = oResultObject.conditions;

				gridObject.disabled = false;
				if (oResultObject.conditions != null)
					if (oResultObject.conditions.enabled != null)
						gridObject.disabled = !oResultObject.conditions.enabled;

				if (oResultObject.implicitRestore != null)
					gridObject.implicitRestore = oResultObject.implicitRestore;

				gridObject.triggers = [];
				for ( var oKey in oResultObject.triggers)
					gridObject.triggers.push(oResultObject.triggers[oKey]);
				for (var iIndex = 0; iIndex < gridObject.triggers.length; iIndex++) {
					if (gridObject.triggers[iIndex].type == 'bus-zone-scene')
						gridObject.triggers[iIndex].type = 'zone-scene';
				}

				gridObject.action = oResultObject.action;
				if (gridObject.action.contentSrc!=null)
					gridObject.action.contentSrc=dss.unescapeProperty(gridObject.action.contentSrc);
				if (oResultObject.target != null)
					if (oResultObject.target.singularyZone != null) {
						gridObject.target = {
							singularyZone : []
						};
						for ( var sKey in oResultObject.target.singularyZone)
							gridObject.target.singularyZone.push(oResultObject.target.singularyZone[sKey]);
					} else {
						gridObject.target = {
							grouped : []
						};
						for ( var sKey in oResultObject.target.grouped)
							gridObject.target.grouped.push(oResultObject.target.grouped[sKey]);
					}

				var fSkip = false;

				if (gridObject.target.singularyZone != null)
					if (gridObject.target.singularyZone.length == 1)
						if (gridObject.triggers.length > 0)
							if (gridObject.triggers[0].type == 'zone-scene')
								if (gridObject.triggers[0].group == 4)
									if (gridObject.triggers[0].zone == gridObject.target.singularyZone[0])
										fSkip = true;
				if (oResultObject.metaData != null) 
					if (oResultObject.metaData!='') {
						var oMeta = Ext.JSON.decode(dss.unescapeProperty(oResultObject.metaData));
						if (oMeta.visibility != null)
							if (oMeta.visibility == false)
								fSkip = true;
						gridObject.metaData=escape(oResultObject.metaData);
					}
				if (!fSkip)
					gridArray.push(gridObject);

			}
			grid.loadParameter(gridArray);
		}

		
		activityStore=grid.store;
		
		
		grid.setStatus(_('Ready'));
		grid.setDisabled(false);
	},
	waitAndReloadActivities : function(grid, parameter) {
		grid.setStatus(_('Saving ...'));
		grid.setDisabled(true);
		var me = this;
		this.waitDialog = new Ext.LoadMask(Ext.getBody(), {
			msg : _('Saving, please wait ...'),
			indicator : false
		});
		this.waitDialog.show();

		dss.raiseEvent('sonos.config', parameter);
		
	}
});

Ext.apply(Ext.data.SortTypes, {
	asPreset : function(oValue) {
		var nValue = 0;
		if (oValue.indexOf("-") == -1) {
			var aValue = oValue.split(" ");
			if (aValue != null && aValue.length > 1)
				nValue = parseInt("" + aValue[1], 10);
		} else
			nValue = 999;

		return nValue;
	}
});

Ext.onReady(function() {
	dss.buildUpLang([ 'locale/{languageSuffix}/sonos-controller.po' ]);
	var sonosControllerApp = Ext.create('DSS.addon.SonosController');
	sonosControllerApp.initPage();
	startListener();
});
